<?php
namespace Neuron\Person\Controller\Api;

use Neuron\Generic\Result;
use Neuron\Application\Framework\Entity;
use Neuron\Person\Relations;
use Neuron\Person\Relation as RelationModel;

class Relation extends \Neuron\Person\Controller {

    const CODE_FIELD_REQUIRED = 410;
    const CODE_SESSION_EXPIRED = 411;

    public function loadAction(){
        $result = new Result();
        $guid   = $this->getREST()->getGuid();
        if ($guid == "" || empty($guid)) {
            $result->code = self::CODE_FIELD_REQUIRED;
            $result->info = 'token is null';
            return $this->getREST()->getResponse($result);
        }

        $entity = Entity::get($this, $guid);
        $user = $entity->user();
        if (!$user->id) {
            $result->code = self::CODE_SESSION_EXPIRED;
            $result->info = 'Session Expired';
            return $this->getREST()->getResponse($result);
        }

        $userPerson = $this->getUserPerson($user);
        $person = $userPerson['person'];
        $personId = $person['person_id'];
        try {
            $relations = new Relations(RelationModel\Storage::factory($this->getDb(),$this->getConfig()));
            $dataRelations = $relations->load($personId, RelationModel\Storage::LISTBY_SUBJECT);
            if($dataRelations->code == 0){
                $list = [];
                if($dataRelations->data['list']){
                    foreach ($dataRelations->data['list'] as $key => $value) {
                        if($value['last_name'] == null){
                            $value['last_name'] = '';
                        }
                        $list[] = $value;
                    }
                }
                $result->data = array(
                    'rows' => $list,
                    'countTotal' => $dataRelations->data['countTotal'],
                );
            }else{
                $result->data = array(
                    'rows' => [],
                    'countTotal' => 0,
                );
            }
            $result->guid = $guid;
        } catch (\Exception $th) {
            $result->code = 500;
            $result->info = $this->translate("failed loading relations");
        }
		
		$requestPost = array(
			'guid' => $guid,
			'code' => '',
			'info' => '',
			'data' => ''
		); 
		
		$responseResult = array(
			'guid' => $result->guid,
			'code' => $result->code,
			'info' => $result->info,
			'data' => $result->data
		); 
		
		$logparam['subject'] = 'API PERSON LOAD FAMILY';
		$logparam['identity'] = $personId ? $personId : '';
		$logparam['user'] = $person ? $person : '';
		$logparam['description'] = 'Log proses api person load family';
		$logparam['request'] = json_encode($requestPost);
		$logparam['response'] = json_encode($responseResult);
		
		$this->writeLogAction($logparam);
		
        return $this->getREST()->getResponse($result);

    }

    public function saveAction(){
        $result = new Result();
        $guid   = $this->getREST()->getGuid();
        $data   = $this->getREST()->getParameters();
        if ($guid == "" || empty($guid)) {
            $result->code = self::CODE_FIELD_REQUIRED;
            $result->info = 'token is null';
            return $this->getREST()->getResponse($result);
        }

        $entity = Entity::get($this, $guid);
        $user = $entity->user();
        if (!$user->id) {
            $result->code = self::CODE_SESSION_EXPIRED;
            $result->info = 'Session Expired';
            return $this->getREST()->getResponse($result);
        }

        $userPerson = $this->getUserPerson($user);
        $person = $userPerson['person'];
        $personId = $person['person_id'];
        $data['subject'] = $personId;
        $data['created_by'] = $user->id;
        try {
            $relation = new RelationModel(RelationModel\Storage::factory($this->getDb(),$this->getConfig()));
            $result = $relation->create($data);
            $result->guid = $guid;
        } catch (\Exception $th) {
            $result->code = 500;
            $result->info = $this->translate("failed save relation");
        }
		
		$requestPost = array(
			'guid' => $guid,
			'code' => '',
			'info' => '',
			'data' => ''
		); 
		
		$responseResult = array(
			'guid' => $result->guid,
			'code' => $result->code,
			'info' => $result->info,
			'data' => $result->data
		); 
		
		$logparam['subject'] = 'API PERSON SAVE FAMILY';
		$logparam['identity'] = $personId ? $personId : '';
		$logparam['user'] = $person ? $person : '';
		$logparam['description'] = 'Log proses api person save family';
		$logparam['request'] = json_encode($requestPost);
		$logparam['response'] = json_encode($responseResult);
		
		$this->writeLogAction($logparam);
		
        return $this->getREST()->getResponse($result);

    }

    public function deleteAction(){
        $result = new Result();
        $guid   = $this->getREST()->getGuid();
        $data   = $this->getREST()->getParameters();
        if ($guid == "" || empty($guid)) {
            $result->code = self::CODE_FIELD_REQUIRED;
            $result->info = 'token is null';
            return $this->getREST()->getResponse($result);
        }

        $entity = Entity::get($this, $guid);
        $user = $entity->user();
        if (!$user->id) {
            $result->code = self::CODE_SESSION_EXPIRED;
            $result->info = 'Session Expired';
            return $this->getREST()->getResponse($result);
        }

        $userPerson = $this->getUserPerson($user);
        $person = $userPerson['person'];
        $personId = $person['person_id'];
        $data['subject'] = $personId;
        $data['created_by'] = $user->id;
        try {
            $relation = new RelationModel(RelationModel\Storage::factory($this->getDb(),$this->getConfig()));
            $result = $relation->delete($data);
            $result->guid = $guid;
        } catch (\Exception $th) {
            $result->code = 500;
            $result->info = $this->translate("failed delete relation");
        }
		
		$requestPost = array(
			'guid' => $guid,
			'code' => '',
			'info' => '',
			'data' => ''
		); 
		
		$responseResult = array(
			'guid' => $result->guid,
			'code' => $result->code,
			'info' => $result->info,
			'data' => $result->data
		); 
		
		$logparam['subject'] = 'API PERSON SAVE FAMILY';
		$logparam['identity'] = $personId ? $personId : '';
		$logparam['user'] = $person ? $person : '';
		$logparam['description'] = 'Log proses api person save family';
		$logparam['request'] = json_encode($requestPost);
		$logparam['response'] = json_encode($responseResult);
		
		$this->writeLogAction($logparam);
		
        return $this->getREST()->getResponse($result);

    }

}
