<?php

namespace Neuron\Person\Controller;

use Neuron\Actor\User;
use Neuron\Person\Attribute as AttrModel;
use Neuron\Person\Entity;
use Neuron\Person\Entities;
use Neuron\Generic\Result;
use Neuron\Person\Map;
use Neuron\Person\Module;

use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;
use PhpOffice\PhpSpreadsheet\Cell\DataValidation;


class Person extends \Neuron\Person\Controller {
	
	public function listAction(){
		$post 	= $this->getREST()->getParameters();
		if(!$post){
			$post 	= $this->getRequest()->getPost();
		}
		if(isset($post['draw']) && !empty($post['draw'])){
			$datatable 		= true;
			$id 			= null;
			$mode 			= Entity\Storage::LIST_ALL;
		}else{
			$datatable 		= false;
			$mode 			= Entity\Storage::LIST_ALL;
			$id 			= null;
		}
		
		$entities 			= new Entities(Entity\Storage::factory(
			$this->getDb(),
			$this->getConfig()
		));
		$result 			= $entities->load($id,$mode,$post);
		if(!$datatable){
			$result->info = $this->translate($result->info);
			return $this->getREST()->getResponse($result);
		}
	}
	
	public function saveAction(){
		$post 					= $this->getREST()->getParameters();
		$sessowner 				= $this->getSession()->owner();
		$post['created_by'] 	= $sessowner;
		$person = new Entity(Entity\Storage::factory(
				$this->getDb(),
				$this->getConfig()
			)
		);

		$result = $person->saveEx($post);

		$result->info = $this->translate($result->info);
		return $this->getREST()->getResponse($result);
	}

	public function createAction(){
		$post 	= $this->getREST()->getParameters();
		$person = new Entity(Entity\Storage::factory(
			$this->getDb(),
			$this->getConfig()
		));
		$post['created_by'] = $this->getSession()->owner();
		$result = $person->create($post);
		$result->info = $this->translate($result->info);
		return $this->getREST()->getResponse($result);
	}

	public function deleteAction() {
		$storage = \Neuron\Person\Entity\Storage::factory($this->getDb(), $this->getConfig());
		$cperson = new \Neuron\Person\Entity($storage);

		$post = $this->getPostParams();

		$retval = $cperson->delete($post['id']);
		
		$requestPost = array(
			'guid' => '',
			'code' => '',
			'info' => '',
			'data' => $post
		); 

		$logparam['subject'] = 'PERSON DELETE';
		$logparam['identity'] = $this->getSession()->owner();;
		$logparam['data1'] = $retval;
		$logparam['description'] = 'Log proses person delete';
		$logparam['request'] = json_encode($requestPost);
		$logparam['response'] = json_encode(array('success' => $retval));
		
		$this->writeLogAction($logparam);
		
		echo json_encode(array('success' => $retval));
		return $this->getResponse();
	}

	public function loadparamAction() {
		$ptstorage = \Neuron\Person\Type\Storage::factory($this->getDb(), $this->getConfig());
		$cptypes = new \Neuron\Person\Types($ptstorage);
		$psstorage = \Neuron\Person\Status\Storage::factory($this->getDb(), $this->getConfig());
		$cpstatus = new \Neuron\Person\Statuses($psstorage);
		$generic 	= new \Neuron\Person\Generic;

		$cache 			= $generic->getCacheAdapter();

		// Get Types
		$hashType 		= md5(\Neuron\Person\Type::CACHE_HASH);
		if(!$cache->hasItem($hashType)){
			$types = $cptypes->listType(0, 999);
			$cache->setItem($hashType,json_encode($types));
			$typeFrom 	= "Database";
		}else{
			$types = json_decode($cache->getItem($hashType),true);
			$typeFrom 	= 'Cache';
		}
		
		// Get Status
		$hashStatus 	= md5(\Neuron\Person\Status::CACHE_HASH);
		if(!$cache->hasItem($hashStatus)){
			$status = $cpstatus->listStatus(0, 999);
			$cache->setItem($hashStatus,json_encode($status));
			$statusFrom 	= "Database";
		}else{
			$status = json_decode($cache->getItem($hashStatus),true);
			$statusFrom  	= "Cache";
		}

		
		$retval = array(
				'types' => $types, 
				'status' => $status,
				'type_source' => $typeFrom,
				'status_source' => $statusFrom
			);
		return $this->getOutput(json_encode($retval));
	}

	public function loadPersonAction(){
		$storage = \Neuron\Person\Entity\Storage::factory($this->getDb(), $this->getConfig());
		$cpersons = new \Neuron\Person\Entity($storage);

		$post = $this->getPostParams();
		//echo '<pre>'; print_r($post); echo '</pre>';

		$result = $cpersons->loadPerson($post['id'], $post['field']);

		echo json_encode($result);
		return $this->getResponse();
	}

	public function saveMappingUserAction(){
		$retval = '';
		$storage = \Neuron\Person\Entity\Storage::factory($this->getDb(), $this->getConfig());
		$cpersons = new \Neuron\Person\Entity($storage);

		$post = $this->getPostParams();
		
		$cekUser  = $cpersons->checkMapping($post['personId'], $post['userId']);
		$success = true;
		$message = 'Error';
		if(!$cekUser){
			$retval = $cpersons->saveMapping($post);
			if($retval){
				$success = true;
				$message = 'Mapping saved';
			}else{
				$success = false;
				$message = 'Error on save';
			}
		}else{
			$success = false;
			$message = 'User mapping already exists';
		}
		
		$requestPost = array(
			'guid' => '',
			'code' => '',
			'info' => '',
			'data' => $post
		); 

		$logparam['subject'] = 'PERSON SAVE';
		$logparam['identity'] = $this->getSession()->owner();;
		$logparam['data1'] = $retval;
		$logparam['description'] = 'Log proses person save';
		$logparam['request'] = json_encode($requestPost);
		$logparam['response'] = json_encode(array('success'=>$success, 'message'=> $message));
		
		$this->writeLogAction($logparam);
		
		echo json_encode(array('success'=>$success, 'message'=> $message));
		return $this->getResponse();
	}

	public function loadMappingUserAction(){
		$storage = \Neuron\Person\Entity\Storage::factory($this->getDb(), $this->getConfig());
		$cpersons = new \Neuron\Person\Entity($storage);

		$post = $this->getPostParams();

		$dataMapping = $cpersons->loadMapping($post['personId']);
		$success = false;
		$message = 'Not found';
		if($dataMapping){
			$success = true;
			$message = 'Found';
		}
	
		return $this->getOutput(json_encode(array('success'=>$success, 'message'=> $message, 'data'=> $dataMapping)));
		
	}

	public function listUserAction(){
		$storage = \Neuron\Person\Entity\Storage::factory($this->getDb(), $this->getConfig());
		$cpersons = new \Neuron\Person\Entity($storage);

		$post = $this->getPostParams();

		$params = array();

		if(isset($post['draw'])){
			$params = array(
				'search' 		=> $post['search']['value'],
				'limit' 		=> $post['length'],
				'offset' 		=> $post['start'],
				'order' 		=> array(
					'dir' 		=> $post['order'][0]['dir'],
					'column' 	=> $post['columns'][ $post['order'][0]['column'] ]['data']
				)
			);
			if($post['start'] == 0 ){
				$page = 1;
			}else{
				$page 	= $post['start'] / $post['length'] + 1; 
			}
		}else{
			$page = 0;
		}


		$dataMap = $cpersons->listMapping($params);
		$arrMap = array();
		if($dataMap){
			foreach ($dataMap as $item) {
				array_push($arrMap, $item);
			}
		}

		$keyword = $post['search']['value'];
		$datatable = true;
		$mode = \Neuron\Actor\User\Storage::LOADBY_PAGE;
		$users = new \Neuron\Actor\Users(
			\Neuron\Actor\User\Storage::factory(
				$this->getDb(),
				$this->getConfig()
			)
		);

		$params = array(
			'page' => $page ,
			'records' => intval($post['length']),
			'order' => $post['order'],
			'columns' => $post['columns'],
		);
		$result = $users->load($keyword, $mode, $params);

		$listUser = $result->data['list'];
		$countTotal = $result->data['countTotal'];

		$arrUser=array();
	    if($listUser){

	    	foreach ($listUser as $user) {
	    		$user['user_map_id']='';
	    		$user['person_name'] = '';
	    		if($arrMap){
					foreach ($arrMap as $map) {
						if($user['user_id'] == $map['user_id']){
							$user['user_map_id'] = $map['user_map_id'];
							$user['person_name'] = $map['person_name'];
						}
					}
				}
				
				array_push($arrUser, $user);
	    	}
	    }

	   	$retval = array(
            'draw' => $post['draw'],
            'recordsTotal' => $countTotal,
            'recordsFiltered' => $countTotal,
            'data' => $arrUser,
        );
        
	    return $this->getOutput(json_encode($retval));
	}

	public function saveWizardAction(){
		$post = $this->getPostParams();
		
		$dataEmployee = @$post['employee'];
		$dataAttribute = @$post['attribute'];
		$dataContract = @$post['contract'];
		$dataContact = @$post['contact'];
		$this->getSession()->get();

		if($dataEmployee){
			//jika post data emp ada

			$postEmp = array();
			foreach ($dataEmployee as $value) {
				$postEmp[$value['name']] = $value['value'];
			}
			

			$storage = \Neuron\Person\Entity\Storage::factory($this->getDb(), $this->getConfig());

			// Check update or create 
			if (!empty($postEmp['id'])) {
				$update = true;
				$cperson = new \Neuron\Person\Entity($storage);
				$cperson->load($postEmp['id']);

				if($cperson->status_id == 2 && $postEmp['status_id'] == 2){
					echo json_encode(array('success'=>false,'message'=>$this->translate('This person is terminated. Please change status to update data.')));
					return $this->getResponse();
				}

				// Kalo kode nya berubah
				if($postEmp['code'] != $cperson->code){
					$modelPerson  = new \Neuron\Person\Entity($storage);
					$cekDuplicate = $modelPerson->checkPersonCode($postEmp['code']);
					if($cekDuplicate){
						echo json_encode(array('success'=>false,'message'=>'Code already exists'));
						return $this->getResponse();
					}
				}

				if(!isset($postEmp['status_id']) || empty($postEmp['status_id'])){
					$postEmp['status_id'] 	= $cperson->status_id;
				}


				// Check ada mappingnya engga
				$map 		= new Map(Map\Storage::factory($this->getDb(),$this->getConfig()));
				if($map->load($cperson->id,Map\Storage::LOADBY_PERSON_ID)){
					$user_id 		= $map->user_id;
					$user 			= new User(User\Storage::factory($this->getDb(),$this->getConfig()));
					if($user->load($user_id)){
						$module 		= new Module();
						$loadCompany 	= $module->getCompanyId($this->getPublicURL().'/');
						if($loadCompany->code == 0){
							$dataCompany 	= $loadCompany->data;
							$companyId 		= $dataCompany->id;
						}

						$oldData 		= $user->pull();
						$user->email 	= $postEmp['email'];
						$user->code 	= $postEmp['code'];
						$newData 		= $user->pull();
						unset($newData['password']);
						$updateUser 	= $user->update($newData);

						$updateRoute 	= $module->updateMasterData(array(
							'company_id' 	=> @$companyId,
							'newData' 		=> array(
								'user_id' 	=> $newData['code'],
								'email' 	=> $newData['email']
							),
							'oldData' 		=> array(
								'user_id' 	=> $oldData['code'],
								'email' 	=> $oldData['email']
							)
						));

						// Kalo gagal update di route, balikin lagi data di act_usernya
						if(intval($updateRoute->code) != 0){
							$user->update($oldData);
						}
					}

					/* Kalo ngerubah statusnya jadi terminate */
					if(($postEmp['status_id'] == self::STATUS_TERMINATED) && $postEmp['status_id'] != $cperson->status_id){
						$user->terminate();
					}
				}

				if($postEmp['birth_date'] == null){
					$cperson->birth_date = null;
				}else{
					$cperson->birth_date = date('Y-m-d', strtotime($postEmp['birth_date']));
				}
				if(!empty($postEmp['birth_date'])){
					$postEmp['birth_date'] = date('Y-m-d', strtotime($postEmp['birth_date']));
				}else{
					$postEmp['birth_date'] = null;
				}
				$postEmp['created_dtm'] = $cperson->created_dtm;
				$postEmp['created_by'] = $cperson->created_by;
				$cperson = new \Neuron\Person\Entity($storage, $postEmp);
				
				//save data emp
				$retval = $cperson->save($update);
			} else {
				if(empty($postEmp['id'])){
					$postEmp['id'] = null;
				}

				if(!empty($postEmp['birth_date'])){
					$postEmp['birth_date'] = date('Y-m-d', strtotime($postEmp['birth_date']));
				}

				$storagePerson =  \Neuron\Person\Entity\Storage::factory($this->getDb(), $this->getConfig());
				$person = new \Neuron\Person\Entity($storagePerson);
				$cperson  = $person;

				$postEmp['created_by'] = $this->getSession()->owner();

				$result = $person->create($postEmp);

				if($result->code !== $result::CODE_SUCCESS){
					$response =  json_encode(array('success' => false, 'message'=> $result->info));
					echo $response;
					return $this->getResponse();
				}

				// Save contact relation
				foreach($dataContact as $contact){
					$storage = \Neuron\Contact\Entity\Storage::factory($this->getDb(), $this->getConfig());
					$ccontact = new \Neuron\Contact\Entity($storage);
					$crelasi['id'] = $contact['relation_id'];
					$crelasi['contact_id'] = $contact['id'];
					$crelasi['relation_type_id'] = $contact['relation_type_id'];
					$crelasi['ref_code'] = $contact['ref_code'];
					$crelasi['ref_id'] = $person->id;
					$crelasi['start_date'] = date('Y-m-d', strtotime($contact['start_date']));
					$crelasi['end_date'] = !empty($contact['end_date'])? date('Y-m-d', strtotime($contact['start_date'])) : null;
					
					if ($contact['relation_id']) {
						$seq = $ccontact->checkRelation($contact['ref_id'], $contact['relation_type_id']);
						$saverel = $ccontact->saveRelation($crelasi, true);
					} else {
						$seq = $ccontact->checkRelation($contact['ref_id'], $contact['relation_type_id']);
						$crelasi['seq'] = count($seq) + 1;
						$saverel = $ccontact->saveRelation($crelasi, false);
					}
				}
				
				$detail = $result->data['detail'];
				$responseDetail = $detail->pull();
				$createData 	= $result->data;
				
				if($result->code == $result::CODE_SUCCESS){
					$retval = true;
				} else {
					$retval = false;
				}
			}

			
			if($retval){
				//jika savenya berhasil
				//cek post attribute
				if($dataAttribute){
					$postAttr = array();
					foreach ($dataAttribute as $attr) {
						$postAttr[$attr['name']] = $attr['value'];
					}
					$postAttr['person_id']  = $cperson->id;
					$attrModel	= new AttrModel(AttrModel\Storage::factory($this->getDb(),$this->getConfig()));
					$session 	= $this->getSession();
					$resultAttr 	= $attrModel->save($postAttr,$session->owner());
				}

				//cek post contract
				$storage = \Neuron\Employee\Contract\Storage::factory($this->getDb(), $this->getConfig());
				$ccontract = new \Neuron\Employee\Contract($storage);
				if(
					class_exists("\Neuron\Employee\Entity") && 
					(
						!class_exists("\Neuron\Addon\Person\ListAttribute\MappingAttribute") || 
						!class_exists("\Neuron\Addon\Person\PersonOrganization\PersonOrganization")
					)
						
				){
					if($dataContract){
						$postContr = array();
						foreach ($dataContract as $contr) {
							$postContr[$contr['name']] = $contr['value'];
						}
	
						$sessmgt = $this->getSession()->get();
						if(!empty($postContr['emp_contract_id'])){
							$update = true;
						}else{
							$update  = false;
							$postContr['emp_contract_id'] = null;
						}
	
						$postContr['start_date'] = (!empty($postContr['start_date']))? date('Y-m-d', strtotime($postContr['start_date'])) : null;
						$postContr['end_date'] = (!empty($postContr['end_date']))? date('Y-m-d', strtotime($postContr['end_date'])) : null;
						$postContr['evidence_id'] = (!empty($postContr['evidence_id']))? $postContr['evidence_id'] : null;
						$postContr['update_dtm'] = date('Y-m-d H:i:s');
						$postContr['emp_type_id'] = $postEmp['type_id'];
						$postContr['signed_by'] = (isset($sessmgt['user_id']) && !empty($sessmgt['user_id'])) ? $sessmgt['user_id'] : $this->getSession()->owner();
						$postContr['update_by'] = (isset($sessmgt['user_id']) && !empty($sessmgt['user_id'])) ? $sessmgt['user_id'] : $this->getSession()->owner();
						$postContr['employee_id']  = $cperson->id;

						$ccontract->push($postContr);
						$retcontr = $ccontract->save($update);
						if(!$retcontr){
							echo json_encode(array('success'=>false,'message'=>'Error save contract employee'));
							return $this->getResponse();
						}
					}
				}
				
				//init save detail emp
				if(empty($postEmp['detail_id'])){
					$postEmp['detail_id'] = null;
					$updateDet = false;
				}else{
					$updateDet = true;
					$postEmp['person_id'] =  $cperson->id;
					$postEmp['join_date']  = (!empty($postEmp['join_date']))? date('Y-m-d', strtotime($postEmp['join_date'])) : null;
					$postEmp['termination_date'] = (!empty($postEmp['termination_date']))? date('Y-m-d', strtotime($postEmp['termination_date'])) : null;
	
					// $storageDet = \Neuron\Person\Detail\Storage::factory($this->getDb(), $this->getConfig());
					// $cdetnew = new \Neuron\Person\Detail($storageDet);
					// $cdetnew->load($postEmp['person_id'], \Neuron\Person\Detail\Storage::LOADBY_PERSON);

					// $postEmp['about'] = (!isset($postEmp['about']) && @$postEmp['about'] =='')? $cdetnew->about : $postEmp['about'];
					// $postEmp['join_date'] = (!isset($postEmp['join_date']) && @$postEmp['join_date'] =='')? $cdetnew->join_date : date('Y-m-d', strtotime($postEmp['join_date']));
					// $postEmp['termination_date'] = (!isset($postEmp['termination_date']) && @$postEmp['termination_date'] =='')? $cdetnew->termination_date : date('Y-m-d', strtotime($postEmp['termination_date']));
					// $postEmp['pic_official'] = (!isset($postEmp['pic_official']) && @$postEmp['pic_official'] =='')? $cdetnew->pic_official : $postEmp['pic_official'];
					// $postEmp['pic_signature'] = (!isset($postEmp['pic_signature']) && @$postEmp['pic_signature'] =='')? $cdetnew->pic_signature : $postEmp['pic_signature'];
					// $postEmp['flag_geolocation'] = isset($postEmp['flag_geolocation']) ? $postEmp['flag_geolocation'] : 0;
					// $postEmp['flag_attd_evidence'] = isset($postEmp['flag_attd_evidence']) ? $postEmp['flag_attd_evidence'] : 0;
					
					// $cdet = new \Neuron\Person\Detail($storageDet, $postEmp);

					// $retDet = $cdet->save($updateDet);
					// $responseDetail = $cdetnew->pull();
					// if(!$retDet){
					// 	echo json_encode(array('success'=>false,'message'=>'Error save detail employee'));
					// 	return $this->getResponse();
					// }
				}

				//save detail emp
				$return = array('success' => true, 'dataEmployee'=> $cperson->pull(), 'dataEmpDetail'=> $responseDetail);
				if(class_exists('\Neuron\Employee\Entity') && (!class_exists("\Neuron\Addon\Person\ListAttribute\MappingAttribute") || !class_exists("\Neuron\Addon\Person\PersonOrganization\PersonOrganization"))){
					$return['dataContract'] = $ccontract->pull();
				}
				
				$requestPost = array(
					'guid' => '',
					'code' => '',
					'info' => '',
					'data' => $post
				);
				
				$logparam['subject'] = 'PERSON SAVEWIZARD';
				$logparam['identity'] = $this->getSession()->owner();;
				$logparam['data1'] = $return['dataEmployee']['id'];
				$logparam['description'] = 'Log proses person saveWizard';
				$logparam['request'] = json_encode($requestPost);
				$logparam['response'] = json_encode($return);

				$writeLog 	= $this->writeLogAction($logparam);
				
				echo json_encode($return);
				return $this->getResponse();

			}else{
				$storage = \Neuron\Person\Entity\Storage::factory($this->getDb(), $this->getConfig());
				$cperson = new \Neuron\Person\Entity($storage);
				echo json_encode(array('success' => $retval, 'data'=> $cperson->pull()));
				return $this->getResponse();
			}
		}
		return $this->getResponse();
	}
	
	public function uploadAction(){
		$result 		= new Result();
		$post 			= $this->getRequest()->getFiles()->toArray();
		$entity 		= new Entity(Entity\Storage::factory($this->getDb(),$this->getConfig()));
		$result 		= $entity->upload($post,$this->getSession()->owner());
		
		$result->info 	= $this->translate($result->info);
		return $this->getREST()->getResponse($result);
	}
	
	public function downloadFormatAction(){
		$entity 		= new Entity(Entity\Storage::factory($this->getDb(),$this->getConfig()));
		$result 		= $entity->getDownloadFormat();
		return $this->getREST()->getResponse($result);
	}
	
	public function countEmpGenderAction(){
		$storage	 	= \Neuron\Person\Entity\Storage::factory($this->getDb(), $this->getConfig());
		$emp 			= new \Neuron\Person\Entity($storage);
		$result 		= $emp->countEmpGender();
		return $this->getREST()->getResponse($result);
	}

}
	
