<?php

namespace Neuron\Person\Controller;
use Neuron\Application\Framework\Entity;
use Neuron\Person\Entity as PersonEntity;
use Neuron\Person\Map;

class Profile extends \Neuron\Person\Controller {
    const CONTACT_REFERENCE     = "PERSON";

    public function profileAction($user){
        $naf = Entity::get();

        $ctr = $naf->getController(); 
        $sessmgt = $naf->session()->get();
        $active_role = $sessmgt['active_role']['code'];
        $session = $naf->session()->owner();
        $ref_id = null;
        $userId 		= $session;
		$sessionRefId 	= null;

		$personMap  	= new \Neuron\Person\Map(\Neuron\Person\Map\Storage::factory(
			$ctr->getDb(),
			$ctr->getConfig()
		));
        
		$personMap->load($userId, \Neuron\Person\Map\Storage::LOADBY_USER_ID);
		if ($personMap->person_id) {
			$ref_id 		= $sessionRefId = $personMap->person_id;
		}

        /**
		 * ada ref_id di url?
		 * kalo ada berarti tampilin profile dari user yang ada di url
		 * set ref_id person yang akan diloadnya sesuai dengan yang di url
		 */
		if ($user->id) {
            $personMap->load($user->id, \Neuron\Person\Map\Storage::LOADBY_USER_ID);
            if ($personMap->person_id) {
                $ref_id 	= $personMap->person_id;
            }
		}

        /* Cari tau apakah user yang login = pemilik profile? */
		$isOwner 		= $ref_id == $sessionRefId;

        $modelPerson  = new \Neuron\Person\Entity(\Neuron\Person\Entity\Storage::factory(
            $ctr->getDb(),
			$ctr->getConfig()
        ));

        //load data person
        $modelPerson->load($ref_id);
        $dataPerson = array_merge($modelPerson->pull());

        //load data relation
        $relations = new  \Neuron\Person\Relations(\Neuron\Person\Relation\Storage::factory($ctr->getDb(),$ctr->getConfig()));
        $dataRelations = $relations->load($ref_id, \Neuron\Person\Relation\Storage::LISTBY_SUBJECT);
        $dataFamily = @$dataRelations->data['list'];

        //loda data status relation

        $relationStatus = new  \Neuron\Person\Relation\Statuses(\Neuron\Person\Relation\Status\Storage::factory($ctr->getDb(),$ctr->getConfig()));
        $dataRelationStatus = $relationStatus->load(null, \Neuron\Person\Relation\Status\Storage::LIST_ALL);
        $dataStatusRelation = $dataRelationStatus->data['list'];

        //load data job
        $employee = new \Neuron\Employee\Entity(\Neuron\Employee\Entity\Storage::factory($ctr->getDb(),$ctr->getConfig()));
        $employee->load($ref_id, \Neuron\Employee\Entity\Storage::LOADBY_PERSON);
        $detail = $employee->detail();
        $type = $employee->type();
        $job = $employee->job();
        $ohp = $job->ohp();
        $dataEmployee = $employee->pull();
        $dataEmployeeDetail = $detail->pull();
        $dataEmployeeType = $type->pull();
        $dataJob = $ohp->pull(10);
        

        $resolver = $naf->getController()
            ->getEvent()
            ->getApplication()
            ->getServiceManager()
            ->get('Zend\View\Resolver\TemplateMapResolver');

        //bikin view baru + set template sesuai resolver
        $view = new \Zend\View\Model\ViewModel();
        $view->setTemplate('neuron/person/form/profile/content/profile');

        $this->using('jquery-validate', 21, array(
            'https://cdnjs.cloudflare.com/ajax/libs/jquery-validate/1.19.1/jquery.validate.min.js',
        ));

        $view->setVariables([
            'title' => $ctr->translate("Personal information"),
            'dataPerson'=> $dataPerson,
            'dataFamily'=> $dataFamily,
            'dataEmployee'=> $dataEmployee,
            'dataEmployeeDetail'=> $dataEmployeeDetail,
            'dataEmployeeType'=> $dataEmployeeType,
            'dataJob'=> $dataJob,
            'is_owner'=> $isOwner,
            'active_role'=> $active_role,
            'dataStatusRelation'=> $dataStatusRelation,
            'user'  => $user
        ]);

        $renderer = $naf->getController()
        ->getEvent()
        ->getApplication()
        ->getServiceManager()
        ->get('Zend\View\Renderer\PhpRenderer');
        $renderer->setResolver($resolver);
        return $renderer->render($view);

    }

    public function experienceAction($user){
        $naf            = Entity::get();

        $ctr            = $naf->getController(); 
        $sessmgt        = $naf->session()->get();
        $active_role    = $sessmgt['active_role']['code'];
        $sessionOwner   = $naf->session()->owner();
        
        $ref_id = null;
        $userId 		= $user->id;
		$personMap  	= new \Neuron\Person\Map(\Neuron\Person\Map\Storage::factory(
			$ctr->getDb(),
			$ctr->getConfig()
		));

		$personMap->load($userId, \Neuron\Person\Map\Storage::LOADBY_USER_ID);
		if ($personMap->person_id) {
			$ref_id 		= $sessionRefId = $personMap->person_id;
		}

        /* Cari tau apakah user yang login = pemilik profile? */
		$isOwner 		= $sessionOwner == $user->id;

        $modelPerson  = new \Neuron\Person\Entity(\Neuron\Person\Entity\Storage::factory(
            $ctr->getDb(),
			$ctr->getConfig()
        ));

        //load data person
        $modelPerson->load($ref_id);
        $dataPerson = array_merge($modelPerson->pull());

        //load data experience
        $experience = new \Neuron\Person\Experiences(\Neuron\Person\Experience\Storage::factory($ctr->getDb(),$ctr->getConfig()));
        $mExperience = $experience->load($ref_id, \Neuron\Person\Experience\Storage::LISTBY_PERSON_TYPE);
        $mExperience = $mExperience->data['list'];
        
        $dataExperience = [];
        if ($mExperience) {
            foreach ($mExperience as $key) {
                $key['period_start'] = date('F Y', strtotime(substr($key['period_start'], 0, 4).'-'.substr($key['period_start'], 4).'-01'));
                $key['period_end']   = date('F Y', strtotime(substr($key['period_end'], 0, 4).'-'.substr($key['period_end'], 4).'-01'));
                $dataExperience[]    = $key;
            }
        }

         //city
         $city = new \Neuron\Contact\Address\Cities(\Neuron\Contact\Address\City\Storage::factory($ctr->getDb(),$ctr->getConfig()));
         $listCity = $city->load();
         $listCity = $listCity->data;

        $employee = new \Neuron\Employee\Entity(\Neuron\Employee\Entity\Storage::factory($ctr->getDb(),$ctr->getConfig()));
        $employee->load($ref_id, \Neuron\Employee\Entity\Storage::LOADBY_PERSON);
        $dataEmployee = $employee->pull();

        $resolver = $naf->getController()
            ->getEvent()
            ->getApplication()
            ->getServiceManager()
            ->get('Zend\View\Resolver\TemplateMapResolver');

        //bikin view baru + set template sesuai resolver
        $view = new \Zend\View\Model\ViewModel();
        $view->setTemplate('neuron/person/form/profile/content/experience');
        $ctr = $naf->getController(); 

        $this->using('jquery-validate', 21, array(
            'https://cdnjs.cloudflare.com/ajax/libs/jquery-validate/1.19.1/jquery.validate.min.js',
        ));
        $view->setVariables([
            'title' => $ctr->translate("Experience"),
            'dataPerson'=> $dataPerson,
            'dataEmployee'  => $dataEmployee,
            'dataExperience'=> $dataExperience,
            'is_owner'=> $isOwner,
            'active_role'=> $active_role,
            'listCity'=> $listCity
        ]);

        $renderer = $naf->getController()
        ->getEvent()
        ->getApplication()
        ->getServiceManager()
        ->get('Zend\View\Renderer\PhpRenderer');
        $renderer->setResolver($resolver);
        return $renderer->render($view);

    }

    public function educationAction($user){
        $naf            = Entity::get();
        $ctr            = $naf->getController(); 
        $sessmgt        = $naf->session()->get();
        $active_role    = $sessmgt['active_role']['code'];
        $sessionOwner   = $naf->session()->owner();
        $ref_id         = null;

        $userId 		= $user->id;
		$personMap  	= new \Neuron\Person\Map(\Neuron\Person\Map\Storage::factory(
			$ctr->getDb(),
			$ctr->getConfig()
		));


		$personMap->load($userId, \Neuron\Person\Map\Storage::LOADBY_USER_ID);
		if ($personMap->person_id) {
			$ref_id 		= $personMap->person_id;
		}

        /* Cari tau apakah user yang login = pemilik profile? */
		$isOwner 		= $user->id == $sessionOwner;

        $modelPerson  = new \Neuron\Person\Entity(\Neuron\Person\Entity\Storage::factory(
            $ctr->getDb(),
			$ctr->getConfig()
        ));
        //load data person
        $modelPerson->load($ref_id);
        $dataPerson = array_merge($modelPerson->pull());

        //load data education
        $educations = new  \Neuron\Person\Educations(\Neuron\Person\Education\Storage::factory($ctr->getDb(),$ctr->getConfig()));
        $dataEducation = $educations->load($ref_id, \Neuron\Person\Education\Storage::LISTBY_PERSON_TYPE);
        $dataEducation = $dataEducation->data['list'];
        $dataFormal = [];
        $dataInformal = [];
        if($dataEducation){
            foreach ($dataEducation as $edu) {
                if($edu['education_type'] == 'FORMAL'){
                    $dataFormal[] = $edu;
                }else{
                    $dataInformal[] = $edu;
                }
            }
        }

        //load parameter
        $parameter = new  \Neuron\Person\Parameters(\Neuron\Person\Parameter\Storage::factory($ctr->getDb(),$ctr->getConfig()));
        $arr = $parameter->listparam();
		$listEduType = array();
		$listEduLevel = array();
		if ($arr) {
			foreach ($arr as $data) {
				if($data['param_ref_name'] == 'EDUCATION_TYPE'){
					array_push($listEduType, $data);
				}else if($data['param_ref_name'] == 'EDUCATION_LEVEL'){
                    $data['value1'] = $ctr->translate($data['value1']);
					array_push($listEduLevel, $data);
				}
			}
		}

        //city
        $city = new \Neuron\Contact\Address\Cities(\Neuron\Contact\Address\City\Storage::factory($ctr->getDb(),$ctr->getConfig()));
        $listCity = $city->load();
        $listCity = $listCity->data;

        $employee = new \Neuron\Employee\Entity(\Neuron\Employee\Entity\Storage::factory($ctr->getDb(),$ctr->getConfig()));
        $employee->load($ref_id, \Neuron\Employee\Entity\Storage::LOADBY_PERSON);
        $dataEmployee = $employee->pull();

        $resolver = $naf->getController()
            ->getEvent()
            ->getApplication()
            ->getServiceManager()
            ->get('Zend\View\Resolver\TemplateMapResolver');

        //bikin view baru + set template sesuai resolver
        $view = new \Zend\View\Model\ViewModel();
        $view->setTemplate('neuron/person/form/profile/content/education');
        $ctr = $naf->getController(); 

        $this->using('jquery-validate', 21, array(
            'https://cdnjs.cloudflare.com/ajax/libs/jquery-validate/1.19.1/jquery.validate.min.js',
        ));
        $view->setVariables([
            'title' => $ctr->translate("Education"),
            'dataPerson'=> $dataPerson,
            'dataEmployee'  => $dataEmployee,
            'dataFormal'=> $dataFormal,
            'dataInformal'=> $dataInformal,
            'is_owner'=> $isOwner,
            'listEduType'=> $listEduType,
            'listEduLevel'=> $listEduLevel,
            'active_role'=> $active_role,
            'listCity'=> $listCity
        ]);

        $renderer = $naf->getController()
        ->getEvent()
        ->getApplication()
        ->getServiceManager()
        ->get('Zend\View\Renderer\PhpRenderer');
        $renderer->setResolver($resolver);
        return $renderer->render($view);

    }

    public function medicalAction($user){
        $naf            = Entity::get();
        $ctr            = $naf->getController(); 
        $sessmgt        = $naf->session()->get();
        $active_role    = $sessmgt['active_role']['code'];
        $sessionOwner   = $naf->session()->owner();
        $ref_id         = null;

        $userId 		= $user->id;
		$personMap  	= new \Neuron\Person\Map(\Neuron\Person\Map\Storage::factory(
			$ctr->getDb(),
			$ctr->getConfig()
		));


		$personMap->load($userId, \Neuron\Person\Map\Storage::LOADBY_USER_ID);
		if ($personMap->person_id) {
			$ref_id 		= $personMap->person_id;
		}

        /* Cari tau apakah user yang login = pemilik profile? */
		$isOwner 		= $user->id == $sessionOwner;

        $modelPerson  = new \Neuron\Person\Entity(\Neuron\Person\Entity\Storage::factory(
            $ctr->getDb(),
			$ctr->getConfig()
        ));
        //load data person
        $modelPerson->load($ref_id);
        $dataPerson = array_merge($modelPerson->pull());

        //load data education
        $mediacals = new  \Neuron\Person\Medicals(\Neuron\Person\Medical\Storage::factory($ctr->getDb(),$ctr->getConfig()));
        $dataMedical = $mediacals->load($ref_id, \Neuron\Person\Medical\Storage::LISTBY_PERSON_TYPE);
        $dataMedical = @$dataMedical->data['list'];
        $dataMildSymptoms = [];
        $dataSurgery = [];
        $dataAllergy = [];
        $dataHereditary = [];
        if($dataMedical){
            foreach ($dataMedical as $med) {
                if($med['medical_type_id'] == 1){
                    $dataMildSymptoms[] = $med;
                }else if($med['medical_type_id'] == 2){
                    $dataSurgery[] = $med;
                }else if($med['medical_type_id'] == 3){
                    $dataAllergy[] = $med;
                }else{
                    $dataHereditary[] = $med;
                    
                }
            }
        }

        //load parameter
        $parameter = new  \Neuron\Person\Parameters(\Neuron\Person\Parameter\Storage::factory($ctr->getDb(),$ctr->getConfig()));
        $arr = $parameter->listparam();
		$listtype = array();
		if ($arr) {
			foreach ($arr as $data) {
				if($data['param_ref_name'] == 'MEDICAL_TYPE'){
                    $data['value1'] = $ctr->translate($data['value1']);
					array_push($listtype, $data);
				}
			}
		}

        $employee = new \Neuron\Employee\Entity(\Neuron\Employee\Entity\Storage::factory($ctr->getDb(),$ctr->getConfig()));
        $employee->load($ref_id, \Neuron\Employee\Entity\Storage::LOADBY_PERSON);
        $dataEmployee = $employee->pull();

        $resolver = $naf->getController()
            ->getEvent()
            ->getApplication()
            ->getServiceManager()
            ->get('Zend\View\Resolver\TemplateMapResolver');

        //bikin view baru + set template sesuai resolver
        $view = new \Zend\View\Model\ViewModel();
        $view->setTemplate('neuron/person/form/profile/content/medical');
        $ctr = $naf->getController(); 

        $this->using('jquery-validate', 21, array(
            'https://cdnjs.cloudflare.com/ajax/libs/jquery-validate/1.19.1/jquery.validate.min.js',
        ));
        $view->setVariables([
            'title' => $ctr->translate("Medical history"),
            'dataPerson'=> $dataPerson,
            'dataEmployee'=> $dataEmployee,
            'dataMildSymptoms'=> $dataMildSymptoms,
            'dataSurgery'=> $dataSurgery,
            'dataAllergy'=> $dataAllergy,
            'dataHereditary'=> $dataHereditary,
            'listtype'=> $listtype,
            'is_owner'=> $isOwner,
            'active_role'=> $active_role
        ]);

        $renderer = $naf->getController()
        ->getEvent()
        ->getApplication()
        ->getServiceManager()
        ->get('Zend\View\Renderer\PhpRenderer');
        $renderer->setResolver($resolver);
        return $renderer->render($view);

    }

    public function contactAction($user){
        $naf    = Entity::get();
        $sessmgt        = $naf->session()->get();

        $map        = new Map(Map\Storage::factory($naf->getDb(),$naf->base()->getModuleConfig()));
        $map->load($user->id,Map\Storage::LOADBY_USER_ID);

        $ctr        = $naf->getController();
        $resolver   = $ctr->getEvent()
            ->getApplication()
            ->getServiceManager()
            ->get('Zend\View\Resolver\TemplateMapResolver');
        $view = new \Zend\View\Model\ViewModel();
        $view->setTemplate('neuron/person/form/profile/content/contact');
        // $ctr = $naf->getController(); 
        
        $ctr->using('p-loading',22);
        $ctr->using('jquery-validate', 21, array(
            'https://cdnjs.cloudflare.com/ajax/libs/jquery-validate/1.19.1/jquery.validate.min.js',
        ));
        $ctr->using('leaflet',26,[
            'https://cdnjs.cloudflare.com/ajax/libs/leaflet/1.7.1/leaflet.min.css',
            'https://cdnjs.cloudflare.com/ajax/libs/leaflet/1.7.1/leaflet.min.js'
        ]);
        $ctr->using('select2', 25, array(
			'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.10/js/select2.min.js',
			'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.10/js/i18n/id.js',
			'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.10/css/select2.min.css',
			'https://cdn.jsdelivr.net/npm/@ttskch/select2-bootstrap4-theme@1.3.2/dist/select2-bootstrap4.min.css',
        ));

        $ref_id = null;
        if ($map->person_id) {
			$ref_id 		= $map->person_id;
		}

        $employee = new \Neuron\Employee\Entity(\Neuron\Employee\Entity\Storage::factory($ctr->getDb(),$ctr->getConfig()));
        $employee->load($ref_id, \Neuron\Employee\Entity\Storage::LOADBY_PERSON);
        $dataEmployee = $employee->pull();

        // Get Form contact
        $entityView     = new \Neuron\Contact\Entity\View($naf->getController(),"PERSON");
        $crudContact    = $entityView->getCrud('v1.1.0');

        $view->setVariables([
            'title' => $ctr->translate("Contact"),
            'person' => $map->person(),
            'employee'  => $dataEmployee,
            'is_owner' => $user->id == $naf->session()->owner(),
            'active_role' => $sessmgt['active_role']['code'],
            'crud_contact' => $crudContact
        ]);

        $renderer = $naf->getController()
        ->getEvent()
        ->getApplication()
        ->getServiceManager()
        ->get('Zend\View\Renderer\PhpRenderer');
        $renderer->setResolver($resolver);
        return $renderer->render($view);
    }

    public function skillsAction($user)
    {
        $naf            = Entity::get();
        $ctr            = $naf->getController(); 
        $sessmgt        = $naf->session()->get();
        $active_role    = $sessmgt['active_role']['code'];
        $sessionOwner   = $naf->session()->owner();
        $ref_id         = null;

        $userId 		= $user->id;
		$personMap  	= new \Neuron\Person\Map(\Neuron\Person\Map\Storage::factory(
			$ctr->getDb(),
			$ctr->getConfig()
		));


		$personMap->load($userId, \Neuron\Person\Map\Storage::LOADBY_USER_ID);
		if ($personMap->person_id) {
			$ref_id 		= $personMap->person_id;
		}

        /* Cari tau apakah user yang login = pemilik profile? */
		$isOwner 		= $user->id == $sessionOwner;

        $modelPerson  = new \Neuron\Person\Entity(\Neuron\Person\Entity\Storage::factory(
            $ctr->getDb(),
			$ctr->getConfig()
        ));
        //load data person
        $modelPerson->load($ref_id);
        $dataPerson = array_merge($modelPerson->pull());

        $employee = new \Neuron\Employee\Entity(\Neuron\Employee\Entity\Storage::factory($ctr->getDb(),$ctr->getConfig()));
        $employee->load($ref_id, \Neuron\Employee\Entity\Storage::LOADBY_PERSON);
        $dataEmployee = $employee->pull();

        $resolver = $naf->getController()
            ->getEvent()
            ->getApplication()
            ->getServiceManager()
            ->get('Zend\View\Resolver\TemplateMapResolver');

        //bikin view baru + set template sesuai resolver
        $view = new \Zend\View\Model\ViewModel();
        $view->setTemplate('neuron/person/form/profile/content/skill');
        $ctr = $naf->getController(); 

        $this->using('jquery-validate', 21, array(
            'https://cdnjs.cloudflare.com/ajax/libs/jquery-validate/1.19.1/jquery.validate.min.js',
        ));

        $this->using('select2', 26, array(
			'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.10/js/select2.min.js',
			/* ini load sesuai bahasa dari session */
			'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.10/js/i18n/id.js',
			/**/
			'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.10/css/select2.min.css',
			'https://cdn.jsdelivr.net/npm/@ttskch/select2-bootstrap4-theme@1.3.2/dist/select2-bootstrap4.min.css',
		));
        
        $view->setVariables([
            'title' => $ctr->translate("Skills"),
            'dataPerson'=> $dataPerson,
            'dataEmployee'  => $dataEmployee,
            'is_owner'=> $isOwner,
            'active_role'=> $active_role,
        ]);

        $renderer = $naf->getController()
        ->getEvent()
        ->getApplication()
        ->getServiceManager()
        ->get('Zend\View\Renderer\PhpRenderer');
        $renderer->setResolver($resolver);
        return $renderer->render($view);
    }

}
	
