<?php

namespace Neuron\Person\Skill\Storage;

class Mysql extends \Neuron\Db\Storage implements Skeleton {
    protected $_nafManager = null;

    //fungsi lama
    public function listdata($params) {
        $select = $this->select()->from(array('a' => $this->_tables['person_skill']));
        
        if ((!empty($params['person_id']))) {
            $select->where(array('a.person_id' => $params['person_id']));
        }
		
        if(!empty($params['order'][0]['column'])){
                $i = $params['order'][0]['column'];
                $dir = $params['order'][0]['dir'];
                $select->order(array($params['columns'][$i]['data'] . " " . $dir));
        }

        if(isset($params['start'])){
            $select->offset(intval($params['start']));
            $select->limit(intval($params['length']));
        }
		// echo str_replace('"','',$select->getSqlString()); die();
        
        $data = $this->fetchAll($select);
        $dataAll = [];

        if($data) {
            foreach($data as $row) {
                array_push($dataAll, $row);
            }
        }

        $selectC = $this->select()->from(array('a' => $this->_tables['person_skill']));
        $selectC->columns(['total_count' => new \Zend\Db\Sql\Expression('COUNT(*)')]);
        if ((!empty($params['person_id']))) {
            $selectC->where(array('a.person_id' => $params['person_id']));
        }
		
        $dataCount = $this->fetchRow($selectC);

        $total = ($dataCount['total_count'] > 0) ? $dataCount['total_count']:0;
                        
        $result = ['data' => $dataAll, 'total' => $total];

        return $result;
    }

    public function load($ids, $mode = \Neuron\Person\Skill\Storage::LOADBY_ID, $params = array()) {
        $select = $this->select()
                        ->from($this->_tables['person_skill']);

        switch ($mode) {
            case \Neuron\Person\Skill\Storage::LOADBY_CODE:
                $select ->where(array('skill_code' => $ids));
                break;
            case \Neuron\Person\Skill\Storage::LOADBY_PARAMS:
                $select ->where(array('skill_code' => $params['skill_code'], 'person_id'    => $params['person_id']));
                break;
            default:
                $select ->where(array('skill_id' => $ids));
                break;
        }
        
        return $this->fetchRow($select);
    }
	
	public function save(\Neuron\Person\Skill $perSkill, $update = false) {
        if ($update) {
            $update = $this->update()
                ->table($this->_tables['person_skill'])
                ->set($this->__($perSkill->pull()))
                ->where($this->__(array('person_skill_id' => $perSkill->person_skill_id)));

            
            if ($this->execute($update)) {
                return $perSkill->person_skill_id;
            }
          
        } else {
            $perSkill->person_skill_id = null;
            $insert = $this->insert()
                        ->into($this->_tables['person_skill'])
                        ->values($this->__($perSkill->pull()));
            $save = $this->execute($insert);
            if ($save) {
                $perSkill->person_skill_id = $save;
                return $perSkill->person_skill_id;
            }
        }

        return false;
    }

    public function remove($perSkillId) {
        $delete = $this->delete()
                        ->from($this->_tables['person_skill'])
                        ->where(array('person_skill_id' => $perSkillId));
                    
        return $this->execute($delete);
    }

    public function loadList($ids, $mode = \Neuron\Person\Skill\Storage::LIST_ALL, $params = null) {
        /* base select */
        $select = $this->select()
                ->columns(array('person_skill_id', 'skill_code', 'skill_value', 'person_id'))
                ->from(array('a' => $this->_tables['person_skill']));

        $select ->join(
            array('b' => $this->_tables['person_skill_types']), 
            'a.skill_code=b.skill_code', 
            array('skill_name'),
            'left' 
        );

        /* loadby page */
        if ($mode == \Neuron\Person\Skill\Storage::LISTBY_PAGE) {
			if(isset($params['person_id']) && !empty($params['person_id'])){
				$select->where(array($this->_('person_id', 'a.') => $params['person_id']));
			}
            
            /* get pager */
            $pager = \Neuron\Db\Storage\Pager::sanitize($params);
            $select->limit($pager->getLimit())->offset($pager->getOffset())->cache(0);

            /* has keyword? */
            if (!empty($ids)) {
                $select->where->nest()
                    ->like($this->_('skill_code', 'a.'), '%' . trim($ids) . '%')
                ->unnest();
            }

            if (!empty($ids)) {
                $select->where->nest()
                    ->like($this->_('skill_name', 'b.'), '%' . trim($ids) . '%')
                ->unnest();
            }

            $select ->order('person_skill_id DESC');

        } else {
            $pager = new \Neuron\Db\Storage\Pager(1, 999999);
        }

        // echo str_replace('"','',$select->getSqlString()); die();
        try {
            $rows               = $this->fetchAll($select);
            $result             = new \stdClass();
            $result->list       = $rows;
            $result->pager      = $pager;
            $result->countTotal = !empty($rows->total) ? $rows->total : 0;
            return $result;
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }
    }

    private function triggerQueryPerson($query, $params, $caller) {

        /* trigger before query */
        if (($this->_nafManager === null) && defined('NAF_CAPABILITY_ADDON')) {
            if ($this->_nafManager = \Neuron\Application\Framework\Event\Manager::get()) {
                $this->_nafManager->events()->trigger('person_education_query', $query, $params, $caller);
            }
        } else if(defined('NAF_CAPABILITY_ADDON')){
            $this->_nafManager->events()->trigger('person_education_query', $query, $params, $caller);
        }
    }

}